/*______________________________________________________________________________
 *
 * Copyright 2005 NORSYS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 * (2) Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 * (3) The name of the author may not be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE
 *______________________________________________________________________________
 *
 * Created on 27 sept. 2005
 * Author: Arnaud Bailly
 */
package speculoos.config.xml;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import javax.xml.parsers.SAXParserFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;

import speculoos.config.Configurator;
import speculoos.core.Mapper;
import speculoos.core.MapperCollector;
import speculoos.manager.Configure;
import speculoos.manager.MapperConfigurationException;
import speculoos.spi.Source;
import speculoos.utils.StrictMap;
import speculoos.utils.VariableSubstitution;

/**
 * Utility class for configuring a business component mapper with an XML file.
 * <p>
 * This configurator reads in one or more XML streams, either from
 * {@link java.io.InputStream}raw data or from {@link java.io.Reader}character
 * data. It is an instance of SAX content handler. This configurator can
 * delegate handling of certain fragments of the XML file(s) read to
 * sub-systems. This is done by first registering another XMLConfigurator using
 * method {@link #addConfigurator(String,Configurator)}which associates a
 * namespace with an instance of configurator. This configurator will then
 * delegate tags within this namespace to the right configurator. The namespace
 * associated with this configurator is
 * <code>http://norsys.fr/framework-ldap/configuration.dtd</code>.
 * </p>
 * <p>
 * This implantation can handle the following tags:
 * <dl>
 * <dd>mapper-config</dd>
 * <dt>Root tag</dt>
 * </dd>
 * <dd>var</dd>
 * <dt>A variable declaration. Later occurence of this variable will be
 * replaced the given value</dt>
 * </dd>
 * <dd>mapper-subconfig</dd>
 * <dt>Declares the class of a subconfigurator that can later be invoked.
 * Attribute <code>class</code> must be a valid class name which can be
 * instantiated without parameters and attribute <code>namespace</code> a
 * string denoting the namespace of this configurator.</dt>
 * </dd>
 * </dl>
 * </p>
 * <p>
 * Sub-classes can override all content handler methods but not
 * {@link #characters(char[], int, int)} which handles variables substitution.
 * Method {@link #getBuffer()} should be used instead to retrieve text nodes'
 * content.
 * </p>
 * <p>
 * Variables can be used in the source XML file. These variables are syntactic:
 * if <code>var</code> is the name of a variable defined in this configurator,
 * then all occurences of the string <code>${var}</code> in the source XML
 * file are replaced by the value of the variable. All variables substitution
 * occurs in attributes values and in text nodes before they are passed to the
 * subconfigurators.
 * </p>
 * <p>
 * A DTD (<code>base-configuration.dtd</code>) is provided for base tags
 * handled by this configurator but as other fragments may be included, it does
 * only include the few tags directly managed by this class. Other DTDs can be
 * included following the pattern given in <code>framework-ldap.dtd</code>.
 * The parser can be set in validating mode using
 * {@link #setValidating(boolean)} method, in which case all syntax errors will
 * give rise to a parsing exception and stops processing.
 * </p>
 * <p>
 * The various parse methods may be called several times as configuration may be
 * split between various locations. Once the method
 * {@link #configure(Configure)}has been called and has returned correctly,
 * client may assume that configuration state is cleared so calls to this method
 * may be interspersed with calls to <code>parse()</code> methods.
 * </p>
 * 
 * @author nono
 * @version $Id: XMLConfigurator.java 212 2005-11-17 15:19:21Z
 *          /C=FR/ST=Nord/L=Lille/O=Norsys SA/OU=UE/CN=Arnaud
 *          Bailly/emailAddress=abailly@norsys.fr $
 * @see <a href="/doc/models/configuration.dtd">DTD for base tags </a>
 */
public class XMLConfigurator extends DefaultHandler implements Configurator,
		MapperCollector {

	/*
	 * table for variables substitution
	 */
	private Map variables = new HashMap();

	/*
	 * map of subconfigurators
	 */
	private Map /* < String, XMLConfigurator > */configurators = new HashMap();

	/*
	 * Variables substitution handling
	 */
	private VariableSubstitution varhandler = new VariableSubstitution();

	/* current buffer for characters data */
	private StringBuffer buffer = new StringBuffer();

	/*
	 * The namespace
	 */
	private String namespace = "http://norsys.fr/framework-ldap/configuration.dtd";

	private XMLConfigurator parent;

	/* default configuration attributes */
	private Map /* <String,Mapper> */mappers = new StrictMap(new HashMap());

	private Map /* <String,Source> */sources = new StrictMap(new HashMap());

	private Map /* <String,String> */links = new StrictMap(new HashMap());

	/* list of validators in this scope */
	private List collected = new ArrayList();

	protected static final Log log = LogFactory.getLog(XMLConfigurator.class);

	/*
	 * DTD Validation flag
	 */
	private boolean validating = false;

	public XMLConfigurator() {
		// set substitution pattern
		varhandler.setNamedVars(Pattern.compile("(\\#\\{([^#{}]+)\\})"));
	}

	/**
	 * Parse data from the given input stream. Platform encoding is assumed
	 * unless explicitly overriden in the xml file header. The stream must
	 * represent data from a valid xml file.
	 * 
	 * @param is
	 *            an InputStream. Must not be null.
	 * @throws IOException
	 * @throws IOExceptionif
	 *             something gets wrong reading the stream
	 */
	public final void parse(InputStream is) throws XMLConfiguratorException {
		try {
			XMLReader reader = SAXParserFactory.newInstance().newSAXParser()
					.getXMLReader();
			reader.setFeature("http://xml.org/sax/features/namespaces", true);
			reader.setFeature("http://xml.org/sax/features/namespace-prefixes",
					true);
			if (validating) {
				reader.setFeature("http://xml.org/sax/features/validation",
						true);
				reader.setErrorHandler(new ConfiguratorErrorHandler(log));
			}
			reader.setEntityResolver(new VariablesResolver());
			reader.setContentHandler(this);
			if (log.isInfoEnabled())
				log.info("start parsing");
			reader.parse(new InputSource(is));
			if (log.isInfoEnabled())
				log.info("done parsing");
		} catch (Exception e) {
			throw new XMLConfiguratorException(
					"Error while parsing XML stream", e);
		}
	}

	/**
	 * Parse data from the given file. This method calls
	 * {@link #parse(InputStream)}with a constructed FileInputStream
	 * 
	 * @param f
	 *            the File to read. May not be null.
	 * @throws XMLConfiguratorException
	 */
	public final void parse(File f) throws XMLConfiguratorException {
		try {
			if (log.isInfoEnabled())
				log.info("start parsing file " + f);
			parse(new FileInputStream(f));
		} catch (FileNotFoundException e) {
			throw new XMLConfiguratorException("Cannot find file " + f, e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see speculoos.core.core.config.ComponentConfigurator#configure(speculoos.core.core.manager.Configure)
	 */
	public void configure(Configure conf) throws MapperConfigurationException {
		/* delegate to subconfigs */
		for (Iterator it = configurators.values().iterator(); it.hasNext();) {
			((XMLConfigurator) it.next()).configure(conf);
		}
		/* add locals */
		for (Iterator i = mappers.entrySet().iterator(); i.hasNext();) {
			Map.Entry me = (Map.Entry) i.next();
			conf.addMapper((String) me.getKey(), (Mapper) me.getValue());
		}
		for (Iterator i = sources.entrySet().iterator(); i.hasNext();) {
			Map.Entry me = (Map.Entry) i.next();
			conf.addSource((String) me.getKey(), (Source) me.getValue());
		}
		for (Iterator i = links.entrySet().iterator(); i.hasNext();) {
			Map.Entry me = (Map.Entry) i.next();
			conf.link((String) me.getKey(), (String) me.getValue());
		}
		for (Iterator i = variables.entrySet().iterator(); i.hasNext();) {
			Map.Entry me = (Map.Entry) i.next();
			conf.addParameter((String) me.getKey(), (String) me.getValue());
		}
		if (log.isInfoEnabled())
			log.info("done configuring " + conf);
	}

	/**
	 * Returns the actual value of a parameter.
	 * 
	 * @param param
	 *            name of parameter
	 * @return a value or <code>null</code> if this parameter does not exists
	 *         in this configurator.
	 */
	public Object getVariable(String param) {
		return variables.get(param);
	}

	/**
	 * Sets the value of a variable in this configuration.
	 * 
	 * @param var
	 *            parameter name
	 * @param value
	 *            parameter value as a string
	 * @throws MapperConfigurationException
	 *             an exception may be thrown if the parameter does not exist in
	 *             this configuration or if the type of
	 */
	public void setVariable(String var, String value)
			throws MapperConfigurationException {
		this.variables.put(var, value);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see speculoos.core.core.config.ComponentConfigurator#addConfigurator(String,
	 *      speculoos.core.core.config.ComponentConfigurator)
	 */
	public void addConfigurator(String ns, Configurator subconf) {
		this.configurators.put(ns, subconf);
		((XMLConfigurator) subconf).setParent(this);
		if (log.isDebugEnabled())
			log.debug("adding xml configurator with ns " + ns);
	}

	/**
	 * Internal method to allow linking of parent and childrend
	 * 
	 * @param configurator
	 */
	private void setParent(XMLConfigurator configurator) {
		this.parent = configurator;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xml.sax.ContentHandler#startElement(java.lang.String,
	 *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		try {
			if (namespace.equals(uri)) {
				if ("var".equals(localName)) {
					String name = attributes.getValue("name");
					String val = varhandler.replaceVars(variables, attributes
							.getValue("value"));
					setVariable(name, val);
				} else if ("sub-config".equals(localName)) {
					String ns = attributes.getValue("name");
					String cln = varhandler.replaceVars(variables, attributes
							.getValue("class"));
					/* try to create object */
					Class cls = Class.forName(cln);
					Object o = cls.newInstance();
					addConfigurator(ns, (XMLConfigurator) o);
				}
			} else {
				/* clear buffer */
				this.buffer.delete(0, Integer.MAX_VALUE);
				/* try to find namespace */
				XMLConfigurator conf = (XMLConfigurator) configurators.get(uri);
				if (conf == null) {
					/* do nothing */
					return;
				}
				/* replace attributes value according to variables */
				AttributesImpl ai = new AttributesImpl(attributes);
				int ln = ai.getLength();
				for (int i = 0; i < ln; i++) {
					String v = ai.getValue(i);
					ai.setValue(i, varhandler.replaceVars(variables, v));
				}
				/* try something else */
				conf.startElement(uri, localName, qName, ai);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new SAXException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xml.sax.ContentHandler#endElement(java.lang.String,
	 *      java.lang.String, java.lang.String)
	 */
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		try {
			if (!namespace.equals(uri)) {
				/* find suitable config */
				XMLConfigurator conf = (XMLConfigurator) configurators.get(uri);
				if (conf != null)
					conf.endElement(uri, localName, qName);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new SAXException(e);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
	 */
	public final void characters(char[] ch, int start, int length)
			throws SAXException {
		/*
		 * apply variables substitution to character array
		 */
		String s = new String(ch, start, length);
		s = varhandler.replaceVars(variables, s);
		/* append to buffer */
		buffer.append(s);
	}

	/**
	 * Returns the accumulated characters as a String.
	 * 
	 * @return the accumulated content of the tags.
	 */
	public final String getBuffer() {
		if (parent != null)
			return parent.getBuffer();
		else
			return buffer.toString().trim();
	}

	/**
	 * Add a new mapper to this configuration;
	 * 
	 * @param name
	 * @param m
	 */
	protected void add(String name, Mapper m) {
		mappers.put(name, m);
	}

	/**
	 * Return a mapper from its name. 
	 * Useful for compound mappers.
	 * 
	 * @param name
	 * @return
	 */
	protected Mapper getMapper(String name) {
		return (Mapper)mappers.get(name);
	}
	
	/**
	 * Add a new source to this configuration
	 * 
	 * @param name
	 * @param s
	 */
	protected void add(String name, Source s) {
		sources.put(name, s);
	}

	/**
	 * Link a mapper name to a source name.
	 * 
	 * @param mn
	 * @param sn
	 */
	protected void link(String mn, String sn) {
		links.put(mn, sn);
	}

	/**
	 * Resets the configuration of this configurator; All accumulated
	 * configuration is cleared as if the XMLConfigurator has just been created.
	 * 
	 */
	public void resetConfig() {
		this.mappers.clear();
		this.sources.clear();
		this.links.clear();
		this.variables.clear();
		this.configurators.clear();
		if (log.isInfoEnabled())
			log.info("resetting config");
	}

	/**
	 * @return Returns the validating.
	 */
	public boolean isValidating() {
		return validating;
	}

	/**
	 * @param validating
	 *            The validating to set.
	 */
	public void setValidating(boolean validating) {
		this.validating = validating;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see speculoos.core.MapperCollector#addMapper(speculoos.core.Mapper)
	 */
	public void addMapper(Mapper m) {
		collected.add(m);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see speculoos.core.MapperCollector#addMappers(java.util.Collection)
	 */
	public void addMappers(Collection col) {
		collected.addAll(col);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see speculoos.core.MapperCollector#getMappers()
	 */
	public Collection getMappers() {
		return Collections.unmodifiableList(collected);
	}

}
